<?php
/* Copyright (C) 2016-2019 Garcia MICHEL <garcia@soamichel.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

dol_include_once('/pricelist/class/pricelist.class.php');

class ActionsPriceList
{
    protected $db;

    public function __construct($db)
    {
        $this->db = $db;
    }

    public function doActions($parameters, &$object, &$action, $hookmanager)
    {
        global $langs, $user, $confirm, $conf;

        $langs->load('pricelist@pricelist');

        if (version_compare(DOL_VERSION, '4.0.0') >= 0) {
            $object->fetch_thirdparty();
            $client = $object->thirdparty;
        } else {
            $client = $object->client;
        }

        $context = $parameters['currentcontext'];
        if ($context == 'ordercard') { /* COMMANDE */
            if ($action == 'addline' and $user->rights->commande->creer) {
                if (GETPOST('prod_entry_mode') == 'free') {
                    return;
                }

                if (!empty($conf->global->PRICELIST_DO_NOT_OVERWRITE_PRICE_WHEN_ADDING) and GETPOST('price_ht', 'int') != 0) {
                    return;
                }

                $idprod = GETPOST('idprod', 'int');
                $qty = GETPOST('qty');

                $pricelist = new PriceList($this->db);
                $obj = $pricelist->get_price($idprod, $client, $qty);
                if (!is_int($obj)) {
                    if ($obj->price) {
                        $_POST['price_ht'] = price($obj->price);
                    } else {
                        $_POST['remise_percent'] = price($obj->tx_discount);
                    }

                    setEventMessage($langs->trans('PriceListInsert'));
                }
            } elseif (in_array($action, array('updateligne', 'updateline')) and $user->rights->commande->creer) {
                $idprod = GETPOST('productid', 'int');
                if (!$idprod) {
                    return;
                }

                $lineid = GETPOST('lineid', 'int');
                $qty = GETPOST('qty');

                $line = new OrderLine($this->db);
                $line->fetch($lineid);
                if (floatval($line->qty) != floatval($qty)) { // qty updated
                    $pricelist = new PriceList($this->db);
                    $obj = $pricelist->get_price($idprod, $client, $qty);
                    if (!is_int($obj)) {
                        if ($obj->price) {
                            $_POST['price_ht'] = price($obj->price);
                        } else {
                            $_POST['remise_percent'] = price($obj->tx_discount);
                        }

                        setEventMessage($langs->trans('PriceListInsert'));
                    }
                }
            }
        } elseif ($context == 'propalcard') { /* PROPAL */
            if ($action == 'addline' and $user->rights->propal->creer) {
                if (GETPOST('prod_entry_mode') == 'free') {
                    return;
                }

                if (!empty($conf->global->PRICELIST_DO_NOT_OVERWRITE_PRICE_WHEN_ADDING) and GETPOST('price_ht', 'int') != 0) {
                    return;
                }

                $idprod = GETPOST('idprod', 'int');
                $qty = GETPOST('qty');

                $pricelist = new PriceList($this->db);
                $obj = $pricelist->get_price($idprod, $client, $qty);
                if (!is_int($obj)) {
                    if ($obj->price) {
                        $_POST['price_ht'] = price($obj->price);
                    } else {
                        $_POST['remise_percent'] = price($obj->tx_discount);
                    }

                    setEventMessage($langs->trans('PriceListInsert'));
                }
            } elseif (in_array($action, array('updateligne', 'updateline')) and $user->rights->propal->creer) {
                $idprod = GETPOST('productid', 'int');
                if (!$idprod) {
                    return;
                }

                $lineid = GETPOST('lineid', 'int');
                $qty = GETPOST('qty');

                $line = new PropaleLigne($this->db);
                $line->fetch($lineid);
                if (floatval($line->qty) != floatval($qty)) { // qty updated
                    $pricelist = new PriceList($this->db);
                    $obj = $pricelist->get_price($idprod, $client, $qty);
                    if (!is_int($obj)) {
                        if ($obj->price) {
                            $_POST['price_ht'] = price($obj->price);
                        } else {
                            $_POST['remise_percent'] = price($obj->tx_discount);
                        }

                        setEventMessage($langs->trans('PriceListInsert'));
                    }
                }
            }
        } elseif (in_array($context, array('invoicecard', 'invoicereccard'))) { /* FACTURE */
            if ($action == 'addline' and $user->rights->facture->creer) {
                if (GETPOST('prod_entry_mode') == 'free') {
                    return;
                }

                if (!empty($conf->global->PRICELIST_DO_NOT_OVERWRITE_PRICE_WHEN_ADDING) and GETPOST('price_ht', 'int') != 0) {
                    return;
                }

                $idprod = GETPOST('idprod', 'int');
                $qty = GETPOST('qty');

                $pricelist = new PriceList($this->db);
                $obj = $pricelist->get_price($idprod, $client, $qty);
                if (!is_int($obj)) {
                    if ($obj->price) {
                        $_POST['price_ht'] = price($obj->price);
                    } else {
                        $_POST['remise_percent'] = price($obj->tx_discount);
                    }

                    setEventMessage($langs->trans('PriceListInsert'));
                }
            } elseif (in_array($action, array('updateligne', 'updateline')) and $user->rights->facture->creer) {
                $idprod = GETPOST('productid', 'int');
                if (!$idprod) {
                    return;
                }

                $lineid = GETPOST('lineid', 'int');
                $qty = GETPOST('qty');

                $line = new FactureLigne($this->db);
                $line->fetch($lineid);
                if (floatval($line->qty) != floatval($qty)) { // qty updated
                    $pricelist = new PriceList($this->db);
                    $obj = $pricelist->get_price($idprod, $client, $qty);
                    if (!is_int($obj)) {
                        if ($obj->price) {
                            $_POST['price_ht'] = price($obj->price);
                        } else {
                            $_POST['remise_percent'] = price($obj->tx_discount);
                        }

                        setEventMessage($langs->trans('PriceListInsert'));
                    }
                }
            }
        }
    }
}
